/*
 *
 * This file is part of the LXARDOSCOPE package.
 *
 * LXARDOSCOPE is an Arduino based oscilloscope for Linux, using the Xforms library.
 *
 * Copyright (C) 2011 Oskar Leuthold
 * 
 * LXARDOSCOPE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * LXARDOSCOPE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with LXARDOSCOPE; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */

#include "forms.h"
#include "lxforms.h"
#include "lxa.h"
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <math.h>

extern FD_lxa *lxa;
extern base_info BASE;
extern channel_info CHAN;
extern horizontal_info HOR;
extern int n,chunk;
extern float *dx,*dy1,*dy2, *tbuf[2];
//************************************************************************
// This subroutine processes and displays data from the soundcard
//
// Note: there may be functionality in this subroutine which is never
// exercised. It was put in for an experimental version of the 
// file reading feature, which was ultimatley implemented in a more useful
// way. 
//
int plot_sound(FL_OBJECT *xyplot)
{
  int trig_pos=0;
  int ds=BASE.disp_size;
  float mdx[6]={0,0,0,0,0,ds},mdy[6]={0,0,5,-5,0,0};
  float tx[2],ty[2];   //for trigger line
  int channel[2]; 
  int chan;
  float sc;
  int toffset=0;
  char cad[16];
  int xind;  // for mouse pointer
//int n=5*DEFAULT_SAMPLING_RATE+1;            // 5 seconds of data
   float daty[2][5*DEFAULT_SAMPLING_RATE+1];

  channel[0]=HOR.trig_chan; 
  channel[1]=HOR.trig_chan==0?1:0;        // not trig channel
  chan=channel[0];

//  the new stuff
int rstart,tstart,tstop,tshift,datend;
int k,n;
int update=0;
int nch; // number of new chunks to be copied from soundcard
int ech; // number of chunks in excess of 64 
// local copy, to make sure the variable does not change while we are updating
// note that these are the numbers of chunks, not indices
int rch=BASE.rchunk;
int cch=BASE.cchunk;
int tch=BASE.tchunk;

if (rch<=cch) return 0;

nch=rch-cch;
if (nch>60) {     			// we can not catch up more than a full tbuf
							// 64 might be fine, 60 is on safe side 
	cch=rch-60;
	nch=rch-cch;
}
// we can only get data for 6 slots of rbuf for one update cycle (the other 2 might be filling)
if (nch>6) nch=6;

// probably useless ....
if(cch==0) { 
   	for (k=0;k<BASE.nbsamples*64;k++) {
    	tbuf[0][k]=0;
		tbuf[1][k]=0;
	}
	BASE.trig=0;
	BASE.tchunk=0;
}
if(nch>0 && tch+nch<=64) { 
		for (n=0;n<nch;n++) {
			rstart=BASE.nbsamples*((rch-nch+n)%8);
			tstart=(tch+n)*BASE.nbsamples;
			for (k=0;k<BASE.nbsamples;k++)  {
			 tbuf[0][tstart+k]=get_sample(0,rstart+k);
			 tbuf[1][tstart+k]=get_sample(1,rstart+k);
			}
		}
BASE.tchunk=BASE.tchunk+nch;
BASE.cchunk=BASE.cchunk+nch;
datend=BASE.tchunk*BASE.nbsamples;
//fills up to max 262143
}
else if(nch>0 && tch+nch>64) { 
		ech=tch+nch-64;
		if(ech<0) ech=0;
		tstart=ech*BASE.nbsamples;
		for (k=0;k<BASE.nbsamples*(tch-ech);k++) {
			tbuf[0][k]=tbuf[0][tstart+k];
			tbuf[1][k]=tbuf[1][tstart+k];
		}
		for (n=0;n<nch;n++) {
			rstart=BASE.nbsamples*((rch-nch+n)%8);
			tstart=(tch-ech+n)*BASE.nbsamples;
			for (k=0;k<BASE.nbsamples;k++)  {
			 tbuf[0][tstart+k]=get_sample(0,rstart+k);
			 tbuf[1][tstart+k]=get_sample(1,rstart+k);
			}
		}
BASE.tchunk=tch-ech+nch;
BASE.cchunk=BASE.cchunk+nch;
datend=BASE.tchunk*BASE.nbsamples;
BASE.trig=BASE.trig-ech*BASE.nbsamples;
}
else {
	printf("internal error: condition not implemented - return\n");
 	return 0; 
}

if(BASE.trig<0) BASE.trig=0;

    switch (HOR.trig_enb)  {
     case TRIG_OFF:       
		if(ds>datend) {
			ds=datend;
			tstart=0; 
		}
		else tstart=datend-ds;
			// fill dx to full display size, to allow proper scaling
		for (int cont=0;cont<BASE.disp_size;cont++) {
     		dx[cont]=cont*1000.0/BASE.sampling_rate;
    	}
    	if (CHAN.on[chan]) {
        for (int cont=0;cont<ds;cont++) 
            daty[chan][cont]=tbuf[chan][cont+tstart];
    	}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont]=0;
    }
// other channel
    	chan=channel[1];
    	if (CHAN.on[chan]) {
        	for (int cont=0;cont<ds;cont++) 
            	daty[chan][cont]=tbuf[chan][cont+tstart];
    		}
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont]=0;    
    	}
      	break;
//////////////////////////////////////////
     case TRIG_ON:
		update=0;
		if(ds>datend) {
			ds=datend;
			tstart=BASE.trig;
			tshift=HOR.T_pos*BASE.sampling_rate/1000.0;
			tstop=datend-tshift; 
	   	 	if(tstop>datend) tstop=datend; 
        	switch (HOR.trig_mode) {
        	case ASC_TRIG:
				for (k=tstart;k<tstop;k++) {
					if(tbuf[chan][k]<=HOR.trig_level && tbuf[chan][k+1]>HOR.trig_level) {
						if(update==0) {
							trig_pos=k;
							update=1;
							BASE.trig=trig_pos;
						}
						toffset=HOR.T_pos*BASE.sampling_rate/1000.0;
						fl_set_object_color(lxa->indicator_triggered,FL_GREEN,FL_GREEN);
					}
				}
			break;
          	case DESC_TRIG:
				for (k=tstart;k<tstop;k++) {
					if(tbuf[chan][k]>=HOR.trig_level && tbuf[chan][k+1]<HOR.trig_level) {
						if(update==0) {
							trig_pos=k;
							update=1;
							BASE.trig=trig_pos;
						}
						toffset=HOR.T_pos*BASE.sampling_rate/1000.0;
						fl_set_object_color(lxa->indicator_triggered,FL_GREEN,FL_GREEN);
					}
				}
	    	break;
        	}
		ds=datend-trig_pos;
		}

		else  {
			tstart=BASE.trig;
			tshift=HOR.T_pos*BASE.sampling_rate/1000.0;
			tstop=datend-tshift-ds;
	   	 	if(tstop>datend-ds) tstop=datend-ds;
        	switch (HOR.trig_mode) {
        	case ASC_TRIG:
				for (k=tstart;k<tstop;k++) {
					if(tbuf[chan][k]<=HOR.trig_level && tbuf[chan][k+1]>HOR.trig_level) {
						trig_pos=k;
						update=1;
						BASE.trig=trig_pos;
						toffset=HOR.T_pos*BASE.sampling_rate/1000.0;
						fl_set_object_color(lxa->indicator_triggered,FL_GREEN,FL_GREEN);
					}
				}
			break;
          	case DESC_TRIG:
				for (k=tstart;k<tstop;k++) {
					if(tbuf[chan][k]>=HOR.trig_level && tbuf[chan][k+1]<HOR.trig_level) {
						trig_pos=k;
						update=1;
						BASE.trig=trig_pos;
						toffset=HOR.T_pos*BASE.sampling_rate/1000.0;
						fl_set_object_color(lxa->indicator_triggered,FL_GREEN,FL_GREEN);
					}
				}
	    	break;
        	}
		}
if(update==0) {
			tstop=datend-tshift; 
	   	 	if(tstop>datend) tstop=datend; 
        	switch (HOR.trig_mode) {
        	case ASC_TRIG:
				for (k=tstart;k<tstop;k++) {
					if(tbuf[chan][k]<=HOR.trig_level && tbuf[chan][k+1]>HOR.trig_level) {
						if(update==0) {
							trig_pos=k;
							update=1;
							BASE.trig=trig_pos;
						}
						toffset=HOR.T_pos*BASE.sampling_rate/1000.0;
						fl_set_object_color(lxa->indicator_triggered,FL_GREEN,FL_GREEN);
					}
				}
			break;
          	case DESC_TRIG:
				for (k=tstart;k<tstop;k++) {
					if(tbuf[chan][k]>=HOR.trig_level && tbuf[chan][k+1]<HOR.trig_level) {
						if(update==0) {
							trig_pos=k;
							update=1;
							BASE.trig=trig_pos;
						}
						toffset=HOR.T_pos*BASE.sampling_rate/1000.0;
						fl_set_object_color(lxa->indicator_triggered,FL_GREEN,FL_GREEN);
					}
				}
	    	break;
        	}
		ds=datend-trig_pos;
		}

if(update==0) ds=0; 
	for (int cont=0;cont<BASE.disp_size;cont++) {
  	   dx[cont]=(cont+toffset)*1000.0/BASE.sampling_rate;
  	  }
  	  if (CHAN.on[chan]) {
  	      for (int cont=0;cont<ds;cont++) 
  	          daty[chan][cont]=tbuf[chan][cont+trig_pos+toffset];
 	   }
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont]=0;
    }
//    other channel
  	  chan=channel[1];
  	  if (CHAN.on[chan]) {
  	      for (int cont=0;cont<ds;cont++) 
  	          daty[chan][cont]=tbuf[chan][cont+trig_pos+toffset];
  	  }
    	else  {
        for (int cont=0;cont<ds;cont++) daty[chan][cont]=0;
    }
}

// calculate and update channel information
float s;
for (int chan=0; chan<2;chan++) {
   CHAN.max[chan]=-1000000;
   CHAN.min[chan]=1000000;
   CHAN.rms[chan]=0;
   CHAN.avg[chan]=0;
   for (int cont=0; cont<ds; cont++) {
      s=daty[chan][cont];
      if (s<CHAN.min[chan]) CHAN.min[chan]=s;
      if (s>CHAN.max[chan]) CHAN.max[chan]=s;
      CHAN.rms[chan]=CHAN.rms[chan]+s*s;
	  CHAN.avg[chan]=CHAN.avg[chan]+s;
    }
	 if(CHAN.rms[chan]>0) CHAN.rms[chan]=sqrt(CHAN.rms[chan]/ds);
	 else CHAN.rms[chan]=0;
     CHAN.pp[chan]=CHAN.max[chan]-CHAN.min[chan];
     CHAN.avg[chan]=CHAN.avg[chan]/ds;
	}

  // *********** Channel 1 ******************
  sprintf(cad,"%.3f",CHAN.min[0]);
  fl_set_object_label(lxa->text_min_chan1,cad);

  sprintf(cad,"%.3f",CHAN.max[0]);
  fl_set_object_label(lxa->text_max_chan1,cad);

  sprintf(cad,"%.3f",CHAN.pp[0]);
  fl_set_object_label(lxa->text_pp_chan1,cad);

  sprintf(cad,"%.3f",CHAN.avg[0]);
  fl_set_object_label(lxa->text_avg_chan1,cad);

  sprintf(cad,"%.3f",CHAN.rms[0]);
  fl_set_object_label(lxa->text_rms_chan1,cad);

  // *********** Channel 2 ******************
  sprintf(cad,"%.3f",CHAN.min[1]);
  fl_set_object_label(lxa->text_min_chan2,cad);

  sprintf(cad,"%.3f",CHAN.max[1]);
  fl_set_object_label(lxa->text_max_chan2,cad);

  sprintf(cad,"%.3f",CHAN.pp[1]);
  fl_set_object_label(lxa->text_pp_chan2,cad);

  sprintf(cad,"%.3f",CHAN.avg[1]);
  fl_set_object_label(lxa->text_avg_chan2,cad);

  sprintf(cad,"%.3f",CHAN.rms[1]);
  fl_set_object_label(lxa->text_rms_chan2,cad);

//
	xind=0;
	while (BASE.pointer_update) {
	   if(dx[xind]>=BASE.wx) {
			sprintf(cad,"%.2f",daty[0][xind]);
  			fl_set_object_label(lxa->text_mouse_V1,cad);
			sprintf(cad,"%.2f",daty[1][xind]);
  			fl_set_object_label(lxa->text_mouse_V2,cad);
			BASE.pointer_update=0;
		}
		xind++;
		if(xind>ds) BASE.pointer_update=0;
	}

switch (BASE.op_mode) {
   case OPERATION_MODE_NORMAL: {
       mdx[0]=dx[0];
	   mdx[5]=dx[BASE.disp_size-1];
	   fl_set_xyplot_xbounds(xyplot,dx[0],dx[BASE.disp_size-1]);
       fl_set_xyplot_data(xyplot,mdx,mdy,6,"","time(ms)","");
//     want yellow lines for x=0 and y=0:
       fl_add_xyplot_overlay(xyplot,1,mdx,mdy,6,FL_YELLOW);
//
if(HOR.trig_enb) {
	tx[0]=mdx[0];tx[1]=mdx[5];
	ty[0]=ty[1]=CHAN.V_pos[HOR.trig_chan]+HOR.trig_level/CHAN.V_scale[HOR.trig_chan];
fl_set_xyplot_overlay_type(xyplot,4,FL_DASHED_XYPLOT);
fl_add_xyplot_overlay(xyplot,4,tx,ty,2,FL_CYAN);
}
else fl_delete_xyplot_overlay(xyplot,4);
//
	   for (int ch=0; ch<2;ch++) {
			chan=channel[ch];	   
			sc=1.0/CHAN.V_scale[chan];
            if (CHAN.inv[chan]==1) sc*=-1;
            for (int cont=0; cont<ds; cont++) {
	           daty[chan][cont]=CHAN.V_pos[chan]+sc*daty[chan][cont];
	   	    }
       		if (CHAN.on[chan]) {
           		fl_add_xyplot_overlay(xyplot,chan+2,dx,daty[chan],ds,CHAN.color[chan]);
           	 }
       		else fl_delete_xyplot_overlay(xyplot,chan+2);
		}
       break;
     }
     case OPERATION_MODE_ADD: {
         mdx[0]=dx[0];
         mdx[5]=dx[BASE.disp_size-1];
         fl_set_xyplot_xbounds(xyplot,dx[0],dx[BASE.disp_size-1]);
         fl_set_xyplot_data(xyplot,mdx,mdy,6,"","time(ms)","");
//     want yellow lines for x=0 and y=0:
         fl_add_xyplot_overlay(xyplot,1,mdx,mdy,6,FL_YELLOW);
		 float sc0=1.0/CHAN.V_scale[0];
		 float sc1=1.0/CHAN.V_scale[0];
		 if (CHAN.inv[0]==1) sc0*=-1;
		 if (CHAN.inv[1]==1) sc1*=-1;
         for (int cont=0;cont<ds;cont++) {
             daty[0][cont]=CHAN.V_pos[0]+sc0*daty[0][cont]+sc1*daty[1][cont];
         }
         fl_add_xyplot_overlay(xyplot,2,dx,daty[0],ds,CHAN.color[0]);
         fl_delete_xyplot_overlay(xyplot,3);
         break;
     }
     case OPERATION_MODE_XY: {
         chan=channel[0];
         mdx[0]=-5.0;
         mdx[5]=5.0;
         sprintf(cad,"Channel %d --->" ,channel[0]+1);
         fl_set_xyplot_xbounds(xyplot,-5.0,5.0);
	     fl_set_xyplot_alphaxtics(xyplot,
			"@-5.0|@-4.0|@-3.0|-@-2.0|@-1.0|0@0.0|@1.0|+@2.0|@3.0|@4.0|@5.0","");
         fl_set_xyplot_data(xyplot,mdx,mdy,6,"",cad,"");
// want yellow lines for x=0 and y=0:
         fl_add_xyplot_overlay(xyplot,1,mdx,mdy,6,FL_YELLOW); 	   
		 for (int ch=0; ch<2;ch++) {
			chan=channel[ch];	   
			sc=1.0/CHAN.V_scale[chan];
            if (CHAN.inv[chan]==1) sc*=-1;
            for (int cont=0; cont<ds; cont++) {
	           daty[chan][cont]=CHAN.V_pos[chan]+sc*daty[chan][cont];
	   	    }
		 }      
         fl_add_xyplot_overlay(xyplot,2,daty[channel[0]],daty[channel[1]],ds,CHAN.color[channel[0]]);
         fl_delete_xyplot_overlay(xyplot,3);
         break;
     }
}   // switch
  return 1;
}



