/*
 *
 * This file is part of the LXARDOSCOPE package.
 *
 * LXARDOSCOPE is an Arduino based oscilloscope for Linux, using the Xforms library.
 *
 * Copyright (C) 2011 Oskar Leuthold
 * 
 * LXARDOSCOPE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * LXARDOSCOPE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with LXARDOSCOPE; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */

#ifndef FD_lxa_h_
#define FD_lxa_h_

/** Callbacks, globals and object handlers **/
extern void CB_xyplot_display(FL_OBJECT *, long);
extern void CB_button_color_chan1(FL_OBJECT *, long);
extern void CB_check_chan1(FL_OBJECT *, long);
extern void CB_droplist_rate(FL_OBJECT *, long);
extern void CB_droplist_dev(FL_OBJECT *, long);
extern void CB_droplist_trigmode(FL_OBJECT *, long);
extern void CB_droplist_trigchan(FL_OBJECT *, long);
extern void CB_droplist_opmode(FL_OBJECT *, long);
extern void CB_button_close(FL_OBJECT *, long);
extern void CB_button_spectrum(FL_OBJECT *, long);
extern void CB_button_display_setup(FL_OBJECT *, long);
extern void CB_logotipo(FL_OBJECT *, long);
extern void CB_counter_Vpos1(FL_OBJECT *, long);
extern void CB_counter_Vscale1(FL_OBJECT *, long);
extern void CB_counter_Tscale(FL_OBJECT *, long);
extern void CB_counter_Tpos(FL_OBJECT *, long);
extern void CB_button_color_chan2(FL_OBJECT *, long);
extern void CB_check_chan2(FL_OBJECT *, long);
extern void CB_counter_Vpos2(FL_OBJECT *, long);
extern void CB_counter_Vscale2(FL_OBJECT *, long);
extern void CB_timer_refresh(FL_OBJECT *, long);
extern void CB_button_inv_chan1(FL_OBJECT *, long);
extern void CB_button_inv_chan2(FL_OBJECT *, long);
extern void CB_button_run(FL_OBJECT *, long);
extern void CB_check_trigger(FL_OBJECT *, long);
extern void CB_button_about_OK(FL_OBJECT *, long);
extern void CB_thumb_trig_level(FL_OBJECT *, long);
extern void CB_thumb_reference(FL_OBJECT *, long);
extern void CB_report_timing(FL_OBJECT *, long);
extern void CB_trig_channel(FL_OBJECT *, long);
extern void CB_trig_mode(FL_OBJECT *, long);
extern void CB_record(FL_OBJECT *, long);
extern void CB_position(FL_OBJECT *, long);
extern void CB_button_gndcal(FL_OBJECT *, long);
extern void CB_button_pulse(FL_OBJECT *, long);
extern int CB_file(const char *, void *);
extern int CB_at_close(FL_FORM *,void *);


/**** Forms and Objects ****/
typedef struct {
	FL_FORM *lxa;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *xyplot_display;
	FL_OBJECT *button_color_chan1;
	FL_OBJECT *check_chan1;
	FL_OBJECT *droplist_rate;
	FL_OBJECT *droplist_dev;
	FL_OBJECT *droplist_trigmode;
	FL_OBJECT *droplist_trigchan;
	FL_OBJECT *droplist_opmode;
	FL_OBJECT *button_close;
	FL_OBJECT *button_spectrum;
	FL_OBJECT *button_display_setup;
	FL_OBJECT *text_mouse_V1;
	FL_OBJECT *text_mouse_V2;
	FL_OBJECT *text_mouse_T;
	FL_OBJECT *text_avg_chan1;
	FL_OBJECT *text_pp_chan1;
	FL_OBJECT *text_rms_chan1;
	FL_OBJECT *logotipo;
	FL_OBJECT *counter_Vpos1;
	FL_OBJECT *counter_Vscale1;
	FL_OBJECT *text_min_chan1;
	FL_OBJECT *text_max_chan1;
	FL_OBJECT *counter_Tscale;
	FL_OBJECT *thumb_trig_level;
	FL_OBJECT *counter_Tpos;
	FL_OBJECT *text_avg_chan2;
	FL_OBJECT *text_pp_chan2;
	FL_OBJECT *text_min_chan2;
	FL_OBJECT *text_max_chan2;
	FL_OBJECT *text_rms_chan2;
	FL_OBJECT *button_color_chan2;
	FL_OBJECT *check_chan2;
	FL_OBJECT *counter_Vpos2;
	FL_OBJECT *counter_Vscale2;
	FL_OBJECT *timer_refresh;
	FL_OBJECT *button_inv_chan1;
	FL_OBJECT *button_inv_chan2;
	FL_OBJECT *button_run;
	FL_OBJECT *indicator_triggered;
	FL_OBJECT *record;
    FL_OBJECT *check_trigger;
    FL_OBJECT *thumb_reference;
    FL_OBJECT *report_timing;
    FL_OBJECT *trig_channel;
    FL_OBJECT *trig_mode;
	FL_OBJECT *trig_label1;
	FL_OBJECT *trig_label2;
	FL_OBJECT *arrow;
	FL_OBJECT *input;
	FL_OBJECT *counter_Pos;
	FL_OBJECT *button_gndcal;
	FL_OBJECT *button_pulse;
} FD_lxa;

extern FD_lxa * create_form_lxa(void);
typedef struct {
	FL_FORM *about;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *button_about_OK;
} FD_about;

extern FD_about * create_form_about(void);

#endif /* FD_lxa_h_ */


/**********************************************************************
 * The following part is derived from pvaluator.h in XFORMS library
 * required for scaler and thumbdig
 */

#ifndef PVALUATOR_H
#define PVALUATOR_H

typedef struct
{
    double min;			/* minimal value of slider */
    double max;			/* maximal value of slider */
    double val;			/* current value of slider */
    double step;		/* step size             */
    int how_return;
    int draw_type;
    int prec;			/* precision when printing value */
    double start_val;
    double ldelta;		/* "left mouse" step   */
    double rdelta;		/* "right mouse " step   */
    double sstep;		/* small step for counter */
    double lstep;		/* large step for counter */
    FL_Coord x, y, w, h;	/* draw (dbl buffer): adjustment report box */
    float slsize;		/* size of the slider    */
    float norm_val;		/* normalized value between 0 & 1 */
    float oldnorm_val;		/* normalized value between 0 & 1 */
    int oldmx, oldmy;
    FL_VAL_FILTER filter;
    FL_Coord offx;
    FL_Coord offy;
    FL_Coord mx, my, mw, mh;	/* mouse: after adjustment for report box  */
    int mouse, lmouse;		/* part the mouse is on                    */
    FL_Coord ww[5], xx[5], yy[5], hh[5];   /* for counter and thumbdig     */
    int changed;
    unsigned int mouseobj;
    int cross_over;             /* allow cross over                        */
    int button;			/* which button pressed: -2<< -1< >1 >>2   */
}
FL_VALUATOR_SPEC;

enum
{
    COMPLETE_DRAW = 1,
    VALUE_DRAW = 2,
    FOCUS_DRAW = 4,
    VALUATOR_DRAWS = 4
};

#define IsGoodReturn(f)  (f==FL_RETURN_END||f==FL_RETURN_END_CHANGED||\
                          f==FL_RETURN_ALWAYS||f==FL_RETURN_CHANGED)

#endif /* PVALUTOR_H */

/*
 *  private header for counter objects
 *
 */
#ifndef PCOUNTER_H
#define PCOUNTER_H

typedef FL_VALUATOR_SPEC FL_COUNTER_SPEC;

#endif /* PCOUNTER_H */


/*
 * Class THUMBDIG header file
 */

#ifndef FL_THUMBDIG_H
#define FL_THUMBDIG_H

#define FL_THUMBDIG    (FL_USER_CLASS_START + 1)

#define FL_THUMBDIG_BW  2


/***** Defaults *****/

#define FL_THUMBDIG_BOXTYPE	FL_UP_BOX
#define FL_THUMBDIG_COL1		FL_COL1
#define FL_THUMBDIG_COL2		FL_BLUE		/* ct label     */
#define FL_THUMBDIG_LCOL		FL_LCOL		/* ct reporting */
#define FL_THUMBDIG_ALIGN	FL_ALIGN_BOTTOM

/***** Others   *****/

//#define FL_THUMBDIG_BW		(FL_BOUND_WIDTH-1)

/***** Routines *****/
FL_EXPORT FL_OBJECT *fl_create_thumbdig(
		int type,
		FL_Coord x,
		FL_Coord y,
		FL_Coord w,
		FL_Coord h,
		const char *label
		);


FL_EXPORT FL_OBJECT *fl_add_thumbdig(
		int type,
		FL_Coord x,
		FL_Coord y,
		FL_Coord w,
		FL_Coord h,
		const char *label
		);


FL_EXPORT void fl_set_thumbdig_value(
		FL_OBJECT *ob,
		double val
		);

FL_EXPORT void fl_set_thumbdig_bounds(
		FL_OBJECT *ob,
		double min,
		double max
		);

FL_EXPORT void fl_set_thumbdig_step(
		FL_OBJECT *ob,
		double s
		);

FL_EXPORT void fl_set_thumbdig_return(
		FL_OBJECT *ob,
		int how
		);

FL_EXPORT double fl_get_thumbdig_value(
		FL_OBJECT *ob
		);

FL_EXPORT void fl_get_thumbdig_bounds(
		FL_OBJECT *ob,
		double *min,
		double *max
		);

FL_EXPORT void fl_get_thumbdig_step(
		FL_OBJECT *ob,
		float *s
		);

FL_EXPORT void fl_set_thumbdig_filter(
		FL_OBJECT *ob,
		FL_VAL_FILTER filter
		);

FL_EXPORT void fl_set_thumbdig_precision(
		FL_OBJECT *ob,
		int prec
		);
#endif


/*
 * Class Scaler header file
 */
#ifndef SCALER_H
#define SCALER_H

#define FL_SCALER    (FL_USER_CLASS_START + 1)

#define FL_SCALER_BW  2


/***** Defaults *****/

#define FL_SCALER_BOXTYPE	FL_UP_BOX
#define FL_SCALER_COL1		FL_COL1
#define FL_SCALER_COL2		FL_BLUE		/* ct label     */
#define FL_SCALER_LCOL		FL_LCOL		/* ct reporting */
#define FL_SCALER_ALIGN	FL_ALIGN_BOTTOM

/***** Others   *****/

//#define FL_SCALER_BW		(FL_BOUND_WIDTH-1)

/***** Routines *****/
FL_EXPORT FL_OBJECT *fl_create_scaler(
                int type,
		FL_Coord x,
		FL_Coord y,
		FL_Coord w,
		FL_Coord h,
		const char *label
		);


FL_EXPORT FL_OBJECT *fl_add_scaler(
                int type,
		FL_Coord x,
		FL_Coord y,
		FL_Coord w,
		FL_Coord h,
		const char *label
		);


FL_EXPORT void fl_set_scaler_value(
		FL_OBJECT *ob,
		double val
		);

FL_EXPORT void fl_set_scaler_bounds(
		FL_OBJECT *ob,
		double min,
		double max
		);

FL_EXPORT void fl_set_scaler_precision(
		FL_OBJECT *ob,
		int prec
		);

FL_EXPORT void fl_set_scaler_return(
		FL_OBJECT *ob,
		int how
		);

FL_EXPORT double fl_get_scaler_value(
		FL_OBJECT *ob
		);

FL_EXPORT void fl_get_scaler_bounds(
		FL_OBJECT *ob,
		double *min,
		double *max
		);

FL_EXPORT void fl_set_scaler_filter(
		FL_OBJECT *ob,
		FL_VAL_FILTER filter
		);

FL_EXPORT int fl_get_scaler_button(
		FL_OBJECT *ob
		);

#endif

