/*
 *
 * This file is part of the LXARDOSCOPE package.
 *
 * LXARDOSCOPE is an Arduino based oscilloscope for Linux, using the Xforms library.
 *
 * Copyright (C) 2011 Oskar Leuthold
 * 
 * LXARDOSCOPE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * LXARDOSCOPE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with LXARDOSCOPE; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */


#include <string.h>
#include <stdlib.h>
#include "forms.h"
#include "lxforms.h"

#define SPEC FL_COUNTER_SPEC

/* given each component a name. parts are numbered 01 4 23 */
enum
{
    NONE, OB0 = 1, OB1 = 2, OB2 = 4, OB3 = 8, OB4 = 16, OB5 = 32, OB6 = 64, ALL = 127
};

static int timdel;		/* Delay since last value change */

/* Draws a thumbdig */
static void
draw_thumbdig(FL_OBJECT * ob)
{
    char str[64];
    int i, btype[5];
    SPEC *sp = (SPEC *) ob->spec;

    /* Compute boxtypes if pushed */
    for (i = 0; i < 5; i++)
	if (ob->pushed && FL_IS_UPBOX(ob->boxtype) &&
	    (sp->mouseobj & (1 << i)))
	    btype[i] = FL_TO_DOWNBOX(ob->boxtype);
	else
	    btype[i] = ob->boxtype;

    if (btype[4] == FL_UP_BOX)
	btype[4] = FL_DOWN_BOX;

        sp->ww[1] = (int)(0.33f * ob->w);
        sp->ww[2] = 3 * sp->ww[1];
	sp->hh[1] = sp->hh[3] = sp->ww[1];  // want square buttons!
	sp->hh[2] = ob->h - 2 * sp->hh[1];        
	sp->xx[0] = ob->x;
	sp->xx[1] = sp->xx[0] + sp->ww[1];
	sp->xx[2] = sp->xx[1] + sp->ww[1];
	sp->yy[0] = ob->y;
        sp->yy[1] = sp->yy[0] + sp->hh[1];
	sp->yy[2] = sp->yy[1] + sp->hh[2];

    if (sp->filter)
	strcpy(str, sp->filter(ob, sp->val, sp->prec));
    else
	sprintf(str, "%.*f", sp->prec, sp->val);

    /* Only draw the parts that need to be drawn  */
    if ((sp->draw_type & OB0))
    {
	fl_drw_box(btype[0], sp->xx[0], sp->yy[0], sp->ww[1], sp->hh[1],
		   ob->col1, ob->bw);
	fl_drw_text(FL_ALIGN_CENTER,sp->xx[0], sp->yy[0], sp->ww[1], sp->hh[1], 
		    ob->col2, 0, 0, "@8>");
    }
    if ((sp->draw_type & OB1))
    {
	fl_drw_box(btype[1], sp->xx[1], sp->yy[0], sp->ww[1], sp->hh[1],
                   ob->col1, ob->bw);
	fl_drw_text(FL_ALIGN_CENTER, sp->xx[1], sp->yy[0], sp->ww[1], sp->hh[1],
		    ob->col2, 0, 0, "@8>");
    }
    if ((sp->draw_type & OB2))
    {
	fl_drw_box(btype[1], sp->xx[2], sp->yy[0], sp->ww[1], sp->hh[1],
                   ob->col1, ob->bw);
	fl_drw_text(FL_ALIGN_CENTER, sp->xx[2], sp->yy[0], sp->ww[1], sp->hh[1],
		    ob->col2, 0, 0, "@8>");
    }
    if ((sp->draw_type & OB3))
    {
	fl_drw_box(btype[4], sp->xx[0], sp->yy[1],  sp->ww[2], sp->hh[2],
		   ob->col1, ob->bw);
	fl_drw_text(FL_ALIGN_CENTER,sp->xx[0], sp->yy[1],  sp->ww[2], sp->hh[2], 
		    ob->lcol, ob->lstyle, ob->lsize, str);
    }
    if ((sp->draw_type & OB4))
    {
	fl_drw_box(btype[2],sp->xx[0], sp->yy[2], sp->ww[1], sp->hh[3], 
		   ob->col1, ob->bw);
	fl_drw_text(FL_ALIGN_CENTER, sp->xx[0], sp->yy[2], sp->ww[1], sp->hh[3],
		    ob->col2, 0, 0, "@2>");
    }
    if ((sp->draw_type & OB5))
    {
	fl_drw_box(btype[3], sp->xx[1], sp->yy[2], sp->ww[1], sp->hh[3],
		   ob->col1, ob->bw);
	fl_drw_text(FL_ALIGN_CENTER, sp->xx[1], sp->yy[2], sp->ww[1], sp->hh[3],
		    ob->col2, 0, 0, "@2>");
    }
    if ((sp->draw_type & OB6))
    {
	fl_drw_box(btype[3], sp->xx[2], sp->yy[2], sp->ww[1], sp->hh[3],
		   ob->col1, ob->bw);
	fl_drw_text(FL_ALIGN_CENTER, sp->xx[2], sp->yy[2], sp->ww[1], sp->hh[3],
		    ob->col2, 0, 0, "@2>");
    }
    if (sp->draw_type == ALL)
	fl_draw_object_label_outside(ob);

    sp->draw_type = ALL;
}


static void
calc_mouse_obj(FL_OBJECT * ob, FL_Coord mx, FL_Coord my)
{
    SPEC *sp = (SPEC *) ob->spec;

    sp->mouseobj = NONE;

    if (my < ob->y || my > ob->y + ob->h || mx < ob->x || mx > ob->x + ob->w)
	return;

	if (mx < sp->xx[1] && my < sp->yy[1])
	    sp->mouseobj = OB0;
	else if (mx > sp->xx[1] && mx < sp->xx[2] && my < sp->yy[1])
	    sp->mouseobj = OB1;
	else if (mx > sp->xx[2] && my < sp->yy[1])
	    sp->mouseobj = OB2;
	else if (my > sp->yy[1] && my < sp->yy[2])
	    sp->mouseobj = OB3;
	else if (mx < sp->xx[1] && my > sp->yy[2])
	    sp->mouseobj = OB4;
	else if (mx > sp->xx[1] && mx < sp->xx[2] && my > sp->yy[2])
	    sp->mouseobj = OB5;
	else if (mx > sp->xx[2] && my > sp->yy[2])
	    sp->mouseobj = OB6;
}

/* handles an event on ob */
static int
handle_mouse(FL_OBJECT * ob, int event, FL_Coord mx, FL_Coord my)
{
    SPEC *sp = (SPEC *) ob->spec;
    int changeval = 0;

    if (event == FL_RELEASE)
    {
	sp->mouseobj = NONE;
	fl_redraw_object(ob);
	return 0;
    }
    else if (event == FL_PUSH)
    {
	calc_mouse_obj(ob, mx, my);
	if (sp->mouseobj != NONE)
	    changeval = 1;
	timdel = 1;
    }
    else if (event == FL_MOUSE && sp->mouseobj != NONE)
	changeval = (timdel++ > 10 && (timdel & 1) == 0);

    if (changeval)
    {
	double oval = sp->val;

	if (sp->mouseobj == OB0)
	    sp->val += 100*sp->sstep;
	if (sp->mouseobj == OB1)
	    sp->val += 10*sp->sstep;
	if (sp->mouseobj == OB2)
	    sp->val += sp->sstep;
	if (sp->mouseobj == OB4)
	    sp->val -= 100*sp->sstep;
	if (sp->mouseobj == OB5)
	    sp->val -= 10*sp->sstep;
	if (sp->mouseobj == OB6)
	    sp->val -= sp->sstep;
        if (sp->val > sp->max)
	    sp->val=oval;
	if (sp->val < sp->min)
	    sp->val=oval;

	if (sp->val != oval)
	{
	    sp->draw_type = sp->mouseobj | OB3;
	    fl_redraw_object(ob);
	}
    }
    return changeval;
}

/* show which button is active by highlighting the button */
static void
show_focus_obj(FL_OBJECT * ob, FL_Coord mx, FL_Coord my)
{
    SPEC *sp = (SPEC *) ob->spec;
    unsigned int oldobj = sp->mouseobj;

    calc_mouse_obj(ob, mx, my);

    /* if same object, do nothing */
    if (sp->mouseobj == oldobj)
	return;

    if (sp->mouseobj && sp->mouseobj != OB3 && sp->mouseobj != oldobj)
    {
	FL_COLOR old = ob->col1;
	sp->draw_type = sp->mouseobj;
	ob->col1 = FL_MCOL;
	fl_redraw_object(ob);
	sp->draw_type = oldobj;
	ob->col1 = old;
	fl_redraw_object(ob);
    }
    else if ((sp->mouseobj == NONE || sp->mouseobj == OB3) && oldobj)
    {
	sp->draw_type = oldobj;
	fl_redraw_object(ob);
    }
}


/* Handles an event */
static int
handle_thumbdig(FL_OBJECT * ob, int event, FL_Coord mx, FL_Coord my,
	       int key, void *ev)
{
    SPEC *sp = (SPEC *) ob->spec;

    switch (event)
    {
    case FL_DRAW:
	draw_thumbdig(ob);
	break;
    case FL_DRAWLABEL:
	fl_draw_object_label_outside(ob);
	break;
    case FL_PUSH:
	sp->changed = handle_mouse(ob, event, mx, my);

	if (sp->how_return == FL_RETURN_CHANGED && sp->changed)
	{
	    sp->changed = 0;
	    return 1;
	}
	break;
    case FL_MOUSE:
	if (handle_mouse(ob, event, mx, my))
	    sp->changed = 1;

	if (sp->how_return == FL_RETURN_CHANGED && sp->changed)
	{
	    sp->changed = 0;
	    return 1;
	}
	break;
    case FL_RELEASE:
	if (handle_mouse(ob, event, mx, my))
	    sp->changed = 1;
	else if (sp->how_return == FL_RETURN_END_CHANGED && sp->changed)
	    return 1;
	break;
    case FL_ENTER:
    case FL_MOTION:
    case FL_LEAVE:
	show_focus_obj(ob, mx, my);
	break;
    case FL_FREEMEM:
	fl_free(ob->spec);
	break;
    }
    return 0;
}

/*------------------------------*/

/* creates an object */
FL_OBJECT *
fl_create_thumbdig(int type, FL_Coord x, FL_Coord y, FL_Coord w, FL_Coord h,
		  const char *label)
{
    FL_OBJECT *ob;
    SPEC *sp;
    fl_add_button_class(FL_THUMBDIG, draw_thumbdig, 0); 
    ob = fl_create_generic_button(FL_THUMBDIG, type, x, y,w, h, label);
    ob = fl_make_object(FL_THUMBDIG, type, x, y, w, h, label, handle_thumbdig);
    ob->boxtype = FL_THUMBDIG_BOXTYPE;
    ob->col1 = FL_THUMBDIG_COL1;
    ob->col2 = FL_THUMBDIG_COL2;
    ob->align = FL_THUMBDIG_ALIGN;
    ob->lcol = FL_THUMBDIG_LCOL;

    if (ob->bw == FL_BOUND_WIDTH && ob->bw == 3)
	ob->bw = FL_THUMBDIG_BW;

    ob->spec_size = sizeof(SPEC);
    ob->spec = fl_calloc(1, sizeof(SPEC));
    sp = ((SPEC *) (ob->spec));
    sp->min = -1000000.0;
    sp->max = 1000000.0;
    sp->sstep = 0.1;
    sp->lstep = 1.0;
    sp->val = 0.0;
    sp->prec = 3;
    sp->mouseobj = NONE;
    sp->draw_type = ALL;
    sp->how_return = FL_RETURN_END_CHANGED;
    return ob;
}

/* Adds an object */
FL_OBJECT *
fl_add_thumbdig(int type, FL_Coord x, FL_Coord y, FL_Coord w, FL_Coord h,
	       const char *label)
{
    FL_OBJECT *ob;

    ob = fl_create_thumbdig(type, x, y, w, h, label);
    fl_add_object(fl_current_form, ob);
    return ob;
}

void
fl_set_thumbdig_value(FL_OBJECT * ob, double val)
{
    SPEC *sp;


    sp = (SPEC *) ob->spec;
    if (sp->val != val)
    {
	sp->val = val;
	sp->draw_type = (ob->visible && ob->form->visible) ? OB3 : ALL;
	fl_redraw_object(ob);
    }
}

void
fl_get_thumbdig_bounds(FL_OBJECT * ob, double *min, double *max)
{
    *min = ((SPEC *) (ob->spec))->min;
    *max = ((SPEC *) (ob->spec))->max;
}

void
fl_set_thumbdig_bounds(FL_OBJECT * ob, double min, double max)
{
    SPEC *sp;

    sp = (SPEC *) ob->spec;
    if (sp->min != min || sp->max != max)
    {
	sp->min = min;
	sp->max = max;
	fl_redraw_object(ob);
    }
}

void
fl_set_thumbdig_step(FL_OBJECT * ob, double s)
{
    SPEC *sp = (SPEC *) ob->spec;

    if (sp->sstep != s)
    {
	sp->sstep = s;
	fl_redraw_object(ob);
    }
}

void
fl_get_thumbdig_step(FL_OBJECT * ob, float *s)
{
    SPEC *sp = (SPEC *) ob->spec;
    *s = (float)sp->sstep;
}

void
fl_set_thumbdig_precision(FL_OBJECT * ob, int prec)
{
    if (((SPEC *) (ob->spec))->prec != prec)
    {
	((SPEC *) (ob->spec))->prec = prec;
	fl_redraw_object(ob);
    }
}


double
fl_get_thumbdig_value(FL_OBJECT * ob)
{
    return ((SPEC *) (ob->spec))->val;
}

/* Sets whether to return value all the time */
void
fl_set_thumbdig_return(FL_OBJECT * ob, int how)
{
    SPEC *sp = (SPEC *) ob->spec;

    sp->how_return = how;
    if (sp->how_return == FL_RETURN_END)
	sp->how_return = FL_RETURN_END_CHANGED;
    else if (sp->how_return == FL_RETURN_ALWAYS)
	sp->how_return = FL_RETURN_CHANGED;
}

void
fl_set_thumbdig_filter(FL_OBJECT * ob, FL_VAL_FILTER filter)
{
    ((SPEC *) (ob->spec))->filter = filter;
}
